package ru.dbotthepony.kstarbound.client.render

import ru.dbotthepony.kstarbound.client.gl.GLStateTracker
import ru.dbotthepony.kstarbound.client.gl.GLTexture2D
import ru.dbotthepony.kstarbound.client.gl.vertex.QuadTransformers
import ru.dbotthepony.kstarbound.defs.animation.IUVCoordinates
import ru.dbotthepony.kstarbound.defs.animation.SpriteReference

/**
 * Связка текстуры-атласа + координат спрайта на ней
 */
class BoundSprite(
	val reference: SpriteReference,
	val texture: GLTexture2D
) : IUVCoordinates {
	inline val sprite get() = reference.sprite

	/**
	 * Настоящая ширина спрайта, в пикселях
	 */
	inline val width get() = reference.sprite.width(texture.width)

	/**
	 * Настоящая высота спрайта, в пикселях
	 */
	inline val height get() = reference.sprite.height(texture.height)

	override val u0: Float
	override val v0: Float
	override val u1: Float
	override val v1: Float

	init {
		val coords = reference.sprite.compute(texture)

		this.u0 = coords.u0
		this.v0 = coords.v0
		this.u1 = coords.u1
		this.v1 = coords.v1
	}

	fun bind() = texture.bind()
	val transformer = QuadTransformers.uv(u0, v0, u1, v1)
}

/**
 * Создаёт связку текстуры-атласа + координгат спрайта на ней
 */
fun SpriteReference.bind(texture: GLTexture2D): BoundSprite {
	return BoundSprite(this, texture)
}

/**
 * Создаёт связку текстуры-атласа, которая загружается через [GLStateTracker.loadNamedTextureSafe] + координгат спрайта на ней
 */
fun SpriteReference.bind(state: GLStateTracker): BoundSprite {
	return BoundSprite(this, state.loadNamedTextureSafe(path))
}
